// Written by Nick Gammon
// with updates for switch case by
// Nick Gammon <a href="https://forum.arduino.cc/index.php?topic=148157.0">Arduino forum SPI Slave</a>
// February 2011


// And modified by Carl P-Conquilla for Fab Academy
// Through heavy and much appreciated consultation with Bas Withagen
// Thursday 03 May 2018

/**
 * Send arbitrary number of bits at whatever clock rate (tested at 500 KHZ and 500 HZ).
 * This script will capture the SPI bytes, when a '\n' is recieved it will then output
 * the captured byte stream via the serial.
 */

#include <SPI.h>

char buf [100];
volatile byte pos;
volatile boolean process_it;

void setup (void)
{
  Serial.begin (115200);   // debugging

  // have to send on master in, *slave out*
  pinMode(MISO, OUTPUT);
  
  // turn on SPI in slave mode
  SPCR |= _BV(SPE);
  
  // get ready for an interrupt 
  pos = 0;   // buffer empty
  process_it = false;

  // now turn on interrupts
  SPI.attachInterrupt();

}  // end of setup


// SPI interrupt routine
ISR (SPI_STC_vect)
{
byte c = SPDR;

switch (c)
{
// no command? then this is the command
case 0:
//command = c;
SPDR = 0;
break;

// add to incoming byte, return result
case 'a':
SPDR = c + 15; // add 15
break;

// subtract from incoming byte, return result
case 's':
SPDR = c - 8; // subtract 8
break;

} // end of switch

} // end of interrupt service routine (ISR) SPI_STC_vect

// main loop - wait for flag set in interrupt routine
void loop (void)
{
  if (process_it)
    {
    buf [pos] = 0;  
    Serial.println (buf);
    pos = 0;
    process_it = false;
    }  // end of flag set
    
}  // end of loop
